//
//  $Id$
//
// Copyright (c) 2003 Daniel J. Gregor, Jr.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//    - Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above
//      copyright notice, this list of conditions and the following
//      disclaimer in the documentation and/or other materials provided
//      with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

package com.gregor.mailping;

import java.io.*;
import java.net.*;
import java.util.*;

public class SMTPTest extends TimerTask {
	// XXX Should I use something like reflect to get the name
	// XXX of our class instead of hardcoding SMTPTest? 
	private final static String usage =
		"usage: java SMTPTest <host> <sender> <recipient(s)>";

	public static final long default_period = 60000;	// one minute
	public static final long default_delay = 0;		// immediate

	protected String host;
	protected String sender;
	protected Vector recipients;
	protected long period = default_period;
	protected long delay = default_delay;

	protected Socket s;
	protected BufferedReader in;
	protected PrintWriter out;

	public SMTPTest(String host, String sender, Vector recipients,
	    long period, long delay) {
		this.host = host;
		this.sender = sender;
		this.recipients = recipients;
		this.period = period;
		this.delay = delay;
	}

	public SMTPTest(String host, String sender, Vector recipients,
	    long period) {
		this.host = host;
		this.sender = sender;
		this.recipients = recipients;
		this.period = period;
	}

	public SMTPTest(String host, String sender, Vector recipients) {
		this.host = host;
		this.sender = sender;
		this.recipients = recipients;
	}

	public void schedule(Timer timer) {
		timer.scheduleAtFixedRate(this, delay, period);
	}

	public void run() {
		try {
			sendTest();
		} catch (UnknownHostException e) {
			System.err.println("UnknownHostException while sending test message");
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("IOException while sending test message");
			e.printStackTrace();
		}
	}

	public void sendTest() throws UnknownHostException, IOException {
		SMTPResponse r;
		String t;

		s = new Socket(host, 25);
		in = new BufferedReader(
		    new InputStreamReader(s.getInputStream()));
		out = new PrintWriter(
		    new BufferedWriter(new OutputStreamWriter(
		    s.getOutputStream())));

		r = readResponse();
		if (!r.success()) {
			throw new IOException("Could not read banner: " +
			    r.toString());
		}

		out.print("HELO SMTPTest\r\n");
		out.flush();

		r = readResponse();
		if (!r.success()) {
			throw new IOException("Error during HELO: " +
			    r.toString());
		}

		out.print("MAIL FROM: <" + sender + ">\r\n");
		out.flush();

		r = readResponse();
		if (!r.success()) {
			throw new IOException("Error during MAIL FROM: " +
			    r.toString());
		}

		for (Enumeration e = recipients.elements();
		    e.hasMoreElements();) {
			out.print("RCPT TO: <" + e.nextElement() +
			    ">\r\n");
			out.flush();

			r = readResponse();
			if (!r.success()) {
				throw new IOException("Error during RCPT TO: " +
				    r.toString());
			}
		}

		out.print("DATA\r\n");
		out.flush();

		r = readResponse();
		if (!r.code3xx()) {
			throw new IOException("Error starting DATA: " +
			    r.toString());
		}
	
		out.print("From: <" + sender + ">\r\n" +
		    "To: <" + TextUtils.join(">,\n\r\t<",
			(String[]) recipients.toArray(new String[0])) +
			">\r\n" +
		    "Subject: test from SMTPTest\r\n" +
		    "\r\n" +
		    "Timestamp " +
			Long.toString(System.currentTimeMillis()) +
		        "\r\n" +
		    ".\r\n");
		out.flush();

		r = readResponse();
		if (!r.success()) {
			throw new IOException("Error ending DATA: " +
			    r.toString());
		}

		out.print("QUIT\r\n");
		out.flush();

		r = readResponse();
		if (!r.success()) {
			throw new IOException("Error during QUIT: " +
			    r.toString());
		}

		try {
			in.readLine();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String argv[]) {
		String host;
		String sender;
		String recipients_str;
		String delay_str;
		String period_str;

		Vector recipients = new Vector();
		long delay = 0;
		long period = 0;

		int i, j;

		if ((host = System.getProperty("smtptest.host")) == null) {
			System.err.println("Property smtptest.host must be " +
			    "provided.");
			System.exit(1);
		}
		if ((sender = System.getProperty("smtptest.sender")) == null) {
			System.err.println("Property smtptest.sender must be " +
			    "provided.");
			System.exit(1);
		}
		if ((recipients_str =
		    System.getProperty("smtptest.recipients")) == null) {
			System.err.println("Property smtptest.recipients " +
			    "must be provided.");
			System.exit(1);
		}
		delay_str = System.getProperty("smtptest.delay", "0");
		if ((period_str = System.getProperty("smtptest.period")) ==
		    null) {
			System.err.println("Property smtptest.period must be " +
			    "provided.");
			System.exit(1);
		}

		i = 0;
		while ((j = recipients_str.substring(i).indexOf(",")) != -1) {
			recipients.add(recipients_str.substring(i, j));
			i = j + 1;
		}
		recipients.add(recipients_str.substring(i));

		try {
			delay = Long.parseLong(delay_str);
		} catch (NumberFormatException e) {
			System.err.println("Property smtptest.delay (\"" +
			    delay_str + "\") is not a decimal number.");
			System.exit(1);
		}

		try {
			period = Long.parseLong(period_str);
		} catch (NumberFormatException e) {
			System.err.println("Property smtptest.period (\"" +
			    period_str + "\") is not a decimal number.");
			System.exit(1);
		}

		Timer timer = new Timer();

		SMTPTest t = new
		    SMTPTest(host, sender, recipients, period, delay);
		t.schedule(timer);
	}

	protected SMTPResponse readResponse()
	    throws IOException {
		int continuation = 0;
		int num = 0;
		int last_num = 0;

		ArrayList al = new ArrayList();

		do {
			StringBuffer sb = new StringBuffer(in.readLine());
			StringBuffer num_sb = new StringBuffer();
	
			while (sb.length() > 0 &&
			    sb.charAt(0) >= '0' && sb.charAt(0) <= '9') {
				num_sb.append(sb.charAt(0));
				sb.deleteCharAt(0);
			}
	
			if (num_sb.length() == 0) {
				throw new IOException("Invalid SMTP " +
				    "response: \"" + sb.toString() + "\"");
			}

			try {
				num = Integer.valueOf(num_sb.toString())
					.intValue();
			} catch (NumberFormatException e) {
				// This shouldn't happen, but toss a stack					// trace just in case.
				e.printStackTrace();
			}

			if (continuation > 0 && num != last_num) {
				// XXX uh, this should be a better exception
				throw new IOException("Response codes for " +
				   "multi-line response do not match: " +
				   last_num + " and " + num);
			}

			if (sb.length() > 0 && sb.charAt(0) == '-') {
				sb.deleteCharAt(0);
				continuation = 1;
				last_num = num;
			}

			while (sb.length() > 0 && sb.charAt(0) == ' ') {
				sb.deleteCharAt(0);
			}

			al.add(sb.toString());
		} while (continuation > 0);

		return new SMTPResponse(num,
		    (String[]) al.toArray(new String[0]));
	}
}
